"""Use-Cases 'FileSysObjects'

* setUpperTreeSearchPath - Create hierarchical search paths.

* findRelPathInUpperTree - Find the absolute realpath of a relative filepathname.

* getCallerName - Get the name of the caller. 

* getModuleFilePathName - Get the filepathname of a module. 

* getModulePathName - Get the pathname of a module. 

* getSourceFilePathName - Get the source filepathname of the caller. 

* getSourceFuncName - Get the function name of the caller. 

* getSourceLinenumber - Get the line number of the call. 

"""